
import functools
from rq import Worker, Queue, Connection

class RQ_Worker(object):

    def __init__(self, uri=None, name="default", ttl=600, result_ttl=3600, redis=None):
        """
        :param uri: The uri dsn
        :param name: the name of the queue, which will also use it as the queue
        :param ttl: TTL
        :param result_ttl: Result TTL
        :param redis: redis.StrictRedis
        :return:
        """
        self.name = name
        self.ttl = ttl
        self.result_ttl = result_ttl
        self.conn = None

        if redis:
            self.conn = redis
        elif uri:
            self.conn = redis.StrictRedis.from_url(url=uri)

        if self.conn:
            self.q = Queue(name=self.name, connection=self.conn)

    def init_app(self, app):
        uri = app.config.get("RQ_WORKER_URI")
        name = app.config.get("RQ_WORKER_NAME", "default")
        ttl = app.config.get("RQ_WORKER_TTL", 600)
        result_ttl = app.config.get("RQ_WORKER_RESULT_TTL", 7200)
        self.__init__(uri=uri, name=name, ttl=ttl, result_ttl=result_ttl)

    def add_job(self, f, *args, **kwargs):
        """
        Enqueue a function to be executed
        :param f: the function to call
        :param args:
        :param kwargs:
        :return object: the job instance including the id

        :example:
        def my_job(args):
            pass

        rq_worker.add_job(my_job, *args, **args)
        """

        kwargs.setdefault("result_ttl", self.result_ttl)
        kwargs.setdefault("ttl", self.ttl)
        f_name = f
        if callable(f) and f.__module__ == '__main__':
            f_name = f.__name__
        return self.q.enqueue(f_name, *args, **kwargs)

    def get_job(self, job_id):
        """
        Return a job by id
        :param job_id:
        :return Job:

        useful properties:
            is_finished
            is_failed
            is_started
            is_queued
            result

            created_at
            ended_at
            enqueued_at

        useful methods:
            get_status()
        """
        return self.q.fetch_job(job_id)

    def run(self):
        """
        This function run the worker in the background
        This function may need to be Supervised so it stays up
        :return:
        """
        listen = [self.name]
        with Connection(self.conn):
            worker = Worker(list(map(Queue, listen)))
            worker.work()

    def clear(self):
        """
        To empty out all queue
        :return:
        """
        c = self.q.count
        self.q.empty()

    def _job(self, f):
        """
        A decorator to add to function, to automatically enqueue
        :return:

        :example:

        @rq_worker.job
        def my_job(args):
            pass

        """
        @functools.wraps(f)
        def delay(*args, **kwargs):
            return self.add_job(f, *args, **kwargs)
        return delay





import redis_collections

class RedisDataStructure(object):

    def __init__(self, db):
        self.db = db

    def List(self, key):
        return redis_collections.List(key=key, redis=self.db)

    def Dict(self, key):
        return redis_collections.Dict(key=key, redis=self.db)

    def Set(self, key):
        return redis_collections.Set(key=key, redis=self.db)


"""
Webportfolio extras
"""

from setuptools import setup, find_packages

__version__ = "0.2.0"
setup(
    name="webportfolio-extras",
    version=__version__,
    license="MIT",
    author="Mardix",
    author_email='mardix@pylot.io',
    description="WebPortfolio extra packages",
    long_description=__doc__,
    url='http://github.com/mardix/webportfolio-extras/',
    download_url='http://github.com/mardix/webportfolio-extras/tarball/master',
    py_modules=['webportfolio_extras'],
    keywords=["webportfolio", "rq", "queue", "redis"],
    include_package_data=True,
    packages=find_packages(),
    platforms='any',
    install_requires=[
        'rq==0.5.3',
        'redis==2.9.1',
        "redis-collections==0.1.7",
        "rollbar",
        "newrelic",
        "mutagen==1.31"
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
